package packetgen

import (
	"net"

	"github.com/google/gopacket"
)

// Common protocol header sizes to help with metrics
const (
	TCPHeaderSize = 25 // 20 for header + at least 5 for options
	UDPHeaderSize = 8
	IPHeaderSize  = 20
)

type Packet struct {
	Link      gopacket.LinkLayer
	Network   gopacket.NetworkLayer
	Transport gopacket.TransportLayer
	Payload   gopacket.Layer
}

type LayerConfig struct {
	Type string
	Data map[string]any
}

type PacketConfig struct {
	Link      LayerConfig
	Network   LayerConfig
	Transport LayerConfig
	Payload   LayerConfig
}

func (c PacketConfig) Build() (result Packet, err error) {
	if result.Link, err = BuildLinkLayer(c.Link); err != nil {
		return Packet{}, err
	}

	if result.Network, err = BuildNetworkLayer(c.Network); err != nil {
		return Packet{}, err
	}

	if result.Transport, err = BuildTransportLayer(c.Transport, result.Network); err != nil {
		return Packet{}, err
	}

	if result.Payload, err = BuildPayload(c.Payload); err != nil {
		return Packet{}, err
	}

	return result, nil
}

func (p Packet) Serialize(payloadBuf gopacket.SerializeBuffer) (err error) {
	return SerializeLayers(payloadBuf, p.Link, p.Network, p.Transport, p.Payload)
}

func (p Packet) IP() net.IP {
	return p.Network.NetworkFlow().Dst().Raw()
}
